var userModule = (function() {
    var userData = {
        isLogged: false,
        token: null,
        name: null,
        tinder_token: null,
        id: null,
        assignment: null,
        isRunBot: false,
        matchedCount: 0,
        votingPlansCount: 0,
        isPledgeToVote: false
    }
    var tabID = 0;

    var initialMethod = function() {
        chrome.storage.sync.get(["userVolt"], function(items){
            data = _parseJSON(items['userVolt'])
            if (typeof data.token !== 'undefined') {
                userData = data
                if (typeof data.assignment !== 'undefined') {
                    //Already get assignment
                    _processAssignmentDetail(data)
                } else {
                    //Only Logged in
                    _processLoggedData(data)
                }
            } else {
                _processNonLoggedData()
                _getFacebookLoggedSession()
            }
        })
    }

    var initialForBackgroundMethod = function() {
        return new Promise(function (resolve, reject) {
            chrome.storage.sync.get(["userVolt"], function(items){
                userData = _parseJSON(items['userVolt'])
                resolve(userData)
            })
        })
    }

    var logout = function() {
        chrome.storage.sync.set({userVolt: {}}, function() {
            _processNonLoggedData()
        });
    }


    chrome.runtime.onMessage.addListener(
        function(request, sender, sendResponse) {
            if (request.msg === "updateVotingPlansMatchedCount") {
                $(".matchedCount").text(request.data.userData.matchedCount);
                $(".votingPlansCount").text(request.data.userData.votingPlansCount);
            }
            if (request.msg === "logout") {
                logout();
                var message = request.message;
                if (message) alert(message);
            }
        }
    );

    var _parseJSON = function(jsonString) {
        try {
            return JSON.parse(jsonString)
        } catch (err) {
            return {}
        }
    }

    var _processLoggedData = function(data) {
        $(".main-content").load("../page_action/basic_information.html", function() {
            $("#facebookUsername").text(data.name);
            $(".matchedCount").text(data.matchedCount);
            $(".votingPlansCount").text(data.votingPlansCount);
        })
    }

    var _processNonLoggedData = function() {
        $(".main-content").load("../page_action/authentication_required.html");
        $(".testing-box").css("display", "none");
        $(".testing-box-error").css("display", "block");
    }

    var _processAssignmentDetail = function(data) {
        $(".main-content").load("../page_action/assignment_detail.html", function() {
            $("#vanityText").text(data.assignment.vanity);
            $("#runBotSwitch").prop('checked', userData.isRunBot);
            if (userData.isRunBot) {
                $(".btn-get-assignment").attr("disabled", userData.isRunBot)
            }
            $(".btn-pledge-vote").prop('disabled', userData.isPledgeToVote);
            $(".matchedCount").text(data.matchedCount);
            $(".votingPlansCount").text(data.votingPlansCount);
        })
    }

    var _getFacebookLoggedSession = function() {
        _facebookLogin().then(function(me){
            var info = {
                id: me.id,
                token: me.token,
                isLogged: true,
                name: me.name,
                matchedCount: 0,
                votingPlansCount: 0
            }
            userData = info
            chrome.storage.sync.set({userVolt: JSON.stringify(info)}, function() {
                chrome.tabs.remove(tabID);
            });
        })
    }

    var _responseError = function(errMessage) {
        alert(errMessage)
    }

    var _facebookLogin = function() {
        return new Promise(function (resolve, reject) {
            chrome.tabs.create({
                url: 'https://www.facebook.com/v2.6/dialog/oauth?redirect_uri=fb464891386855067%3A%2F%2Fauthorize%2F&state=%7B%22challenge%22%3A%22q1WMwhvSfbWHvd8xz5PT6lk6eoA%253D%22%2C%220_auth_logger_id%22%3A%2254783C22-558A-4E54-A1EE-BB9E357CC11F%22%2C%22com.facebook.sdk_client_state%22%3Atrue%2C%223_method%22%3A%22sfvc_auth%22%7D&scope=user_birthday%2Cuser_photos%2Cuser_education_history%2Cemail%2Cuser_relationship_details%2Cuser_friends%2Cuser_work_history%2Cuser_likes&response_type=token%2Csigned_request&default_audience=friends&return_scopes=true&auth_type=rerequest&client_id=464891386855067&ret=login&sdk=ios&logger_id=54783C22-558A-4E54-A1EE-BB9E357CC11F#_=_',
                active: false
            }, function(tab) {
                // After the tab has been created, open a window to inject the tab
                tabID = tab.id
                chrome.windows.create({
                    tabId: tab.id,
                    type: 'popup',
                    focused: true,
                    width: 700,
                    height: 700,
                    left: (window.screen.availWidth/4) | 0,
                    top: 100
                });


                function getAccessTokenProcess(){

                    var currentTab;
                    var version = "1.0";
                    // https://stackoverflow.com/questions/18534771/chrome-extension-how-to-get-http-response-body


                    function onAttach(tabId) {
                        chrome.debugger.sendCommand({ //first enable the Network
                            tabId: tabId
                        }, "Network.enable");
                        chrome.debugger.onEvent.addListener(allEventHandler);

                    }


                    function allEventHandler(debuggeeId, message, params) {
                        if (message == "Network.responseReceived") { //response return
                            chrome.debugger.sendCommand({
                                tabId: debuggeeId.tabId
                            }, "Network.getResponseBody", {
                                "requestId": params.requestId
                            }, function(response) {
                                if(chrome.runtime.lastError) {
                                    //This is for remove the console err
                                }
                                // you get the response body here!
                                // you can close the debugger tips by:
                                if(typeof response !== 'undefined') {
                                    //Get access_token
                                    response = JSON.stringify(response)
                                    var re = /&access_token=(.*)&expires/;
                                    var result = re.exec(response);
                                    if (result !== null) {
                                        var access_token = result[1]

                                        chrome.debugger.detach(debuggeeId);
                                        //Get Facebook userID and name
                                        fetch('https://graph.facebook.com/v2.3/me?access_token=' + access_token)
                                            .then(function (r) {
                                                return r.json()
                                            })
                                            .then(function (me) {
                                                me.token = access_token
                                                resolve(me)
                                            })
                                            .catch(function(err) {
                                                //TODO: Friendly error handling.
                                                _responseError(err)
                                                return reject('rejected')
                                            })
                                    }
                                }
                            });
                        }

                    }


                    setTimeout(function(){

                        chrome.tabs.query({'active': true, 'lastFocusedWindow': true}, function (tabs) {
                            if (typeof tabs[0] === 'undefined') {
                                var url = "null";
                            } else {
                                currentTab = tabs[0].id;
                                chrome.debugger.attach({ //debug at current tab
                                    tabId: currentTab
                                }, version, onAttach.bind(null, currentTab));
                            }
                        });
                    }, 500)
                }
                getAccessTokenProcess()
            });


        })
    }

    var getAssignmentMethod = function() {
        return new Promise(function (resolve, reject) {
            fetch('./getassignment')
                .then(function (r) {
                    return r.json()
                })
                .then(function (me) {
                    userData.assignment = me.Item;
                    userData.isRunBot = true;
                    _processAssignmentDetail(userData)
                    chrome.storage.sync.set({userVolt: JSON.stringify(userData)}, function() {
                        chrome.runtime.sendMessage({mode: "setRunBot", isRunBot: true}, function(response) {
                            return resolve(userData)
                        });

                    });


                })
                .catch(function(err) {
                  //TODO: Friendly error handling.
                    _responseError(err)
                    return reject('rejected')
                })
        })
    }

    var setBotStateMethod = function(state) {
        return new Promise(function (resolve, reject) {
            userData.isRunBot = state
            chrome.storage.sync.set({userVolt: JSON.stringify(userData)}, function() {
                return resolve(userData)
            });
        })
    }

    var setMatchedCount = function(total) {
        return new Promise(function (resolve, reject) {
            if (!userData.token) {
                reject("User is not initialized")
            } else {
                userData.matchedCount = total
                chrome.storage.sync.set({userVolt: JSON.stringify(userData)}, function() {
                    _renderMatchesVotingPlans();
                    resolve(userData)
                });
            }
        })
    }

    var setVotingPlans = function(total) {
        return new Promise(function (resolve, reject) {
            if (!userData.token) {
              //TODO: Pull out user messages to enable localization.
                reject("User is not initialized")
            } else {
                userData.votingPlansCount = total
                chrome.storage.sync.set({userVolt: JSON.stringify(userData)}, function() {
                    _renderMatchesVotingPlans();
                    resolve(userData)
                });
            }
        })
    }

    var sendLogging = function(requestData) {
        return new Promise(function (resolve, reject) {
            if (requestData.meta == META.MSSG_SENT || requestData.meta == META.MSSG_RECEIVED) {
                if (!requestData.tinderID || !requestData.targetTinderID || !requestData.meta || !requestData.message) {
                  //TODO: Pull out user messages to enable localization.
                    reject("Missing params: please check tinderID, targetTinderID, meta, message");
                }

                if (!userData.assignment.aID) {
                  //TODO: Pull out user messages to enable localization.
                    reject("userData assignment is not initialized");
                }
            }

            requestData.aID = userData.assignment.aID;

            fetch(ENV.API_URL+'/latest/reportstats', {
                method: 'post',
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify(requestData)
            }).then(function(response) {
                if (requestData.meta == META.MSSG_SENT || requestData.meta == META.MSSG_RECEIVED) {
                    _updateVotingPlans(requestData.message.message).then(function(res) {
                        _renderMatchesVotingPlans();
                        resolve(requestData)
                    })
                } else {
                    resolve(requestData)
                }
            }).catch(function(err) {
                reject(err);
            });
        })
    }
    //TODO: Voter pledges need a partner and tracking capabilities.

    var _updateVotingPlans = function(message) {
        return new Promise(function (resolve, reject) {
            _checkCountVotingPlans(message).then(function(res) {
                if (res) {
                    if (!userData.votingPlansCount) userData.votingPlansCount = 0;
                    userData.votingPlansCount = parseInt(userData.votingPlansCount) + 1;
                    chrome.storage.sync.set({userVolt: JSON.stringify(userData)}, function() {
                        _renderMatchesVotingPlans();
                        resolve(userData)
                    });
                } else {
                    resolve(false)
                }
            })
        })
    }

    var _checkCountVotingPlans = function(message) {
        return new Promise(function (resolve, reject) {
            console.log(message)
            // var re = /https:\/\/www.vote.org\/register-to-vote\//;
            var re = /Counting/;
            var result = re.exec(message);
            if (result !== null) {
                resolve(true)
            } else {
                resolve(false)
            }
        })
    }

    var _renderMatchesVotingPlans = function() {
        $(".matchedCount").text(userData.matchedCount);
        $(".votingPlansCount").text(userData.votingPlansCount);
    }

    var increasePledgeToVote = function() {
        return new Promise(function (resolve, reject) {
            if (!userData.isPledgeToVote) {
                userData.votingPlansCount = parseInt(userData.votingPlansCount) + 1;
                userData.isPledgeToVote = true;
                $(".btn-pledge-vote").prop('disabled', userData.isPledgeToVote);
                chrome.storage.sync.set({userVolt: JSON.stringify(userData)}, function() {
                    _renderMatchesVotingPlans();
                    resolve(userData)
                });
            } else {
                resolve(userData)
            }
        })
    }

    return {
        initial: initialMethod,
        initialForBackground: initialForBackgroundMethod,
        userData: function() { return userData; },
        getAssignment: getAssignmentMethod,
        setBotState: setBotStateMethod,
        setMatchedCount: setMatchedCount,
        setVotingPlans: setVotingPlans,
        sendLogging: sendLogging,
        increasePledgeToVote: increasePledgeToVote,
        logout: logout
    }
})()
