var tinderModule = (function() {

    var client = null;

    var initialMethod = function(TinderClient) {
        client = new TinderClient.TinderClient();
    }

    var tinderAuthentication = function() {
        return new Promise(function(resolve, reject) {
            var userData = userModule && userModule.userData();
            if (userData && userData.token && userData.token != "" && client) {
                client.authorize(
                    userData.token,
                    userData.id,
                    function(err, data) {
                        if (err) {
                            //Check if current loggin session is expired
                            var re = /Session has expired|Failed to authenticate|Unauthorized/;
                            var isExpired = re.exec(err);
                            if (isExpired !== null) {
                                //TODO: Automatically renew session.
                                alert("Your logged session has expired, please login again!")
                            }
                            userModule.logout();
                        } else {
                            resolve(data);
                        }
                    }
                );
            }
        })
    }

    var isAuthorized = function() {
        if (client) {
            return client.isAuthorized()
        }
    }

    var setLocation = function(callback) {
        if (userModule.userData().token && userModule.userData().assignment != "") {
            var longitude = userModule.userData().assignment.long;
            var latitude = userModule.userData().assignment.lat;
            client.updatePosition(longitude, latitude, callback)
        }
    }

    var authorize = function() {
        if (userModule.userData().token && userModule.userData().token != "") {
            client.authorize(
                userModule.userData().token,
                userModule.userData().id,
                function(err) {
                    if (err) console.log(err);
                }
            );
        }
    }

    var like = function(id, callback) {
        if (client) {
            client.like(id, callback)
        }
    }

    var unlike = function(id, callback) {
        if (client) {
            client.unmatch(id, callback)
        }
    }

    var sendMessage = function(id, message, callback) {
        if (client) {
            client.sendMessage(id, message, callback)
        }
    }

    var getMessages = function(matchedId, count, callback) {
        if (client) {
            client.getMessages(matchedId, count, callback)
        }
    }

    var getMatches = function(count, haveMessage, callback) {
        if (client) {
            client.getMatches(count, haveMessage, callback)
        }
    }

    var getUserId = function() {
        if (client) {
            return client.userId
        }
    }

    var getRecommendations = function(limit, callback) {
        if (client) {
            client.getRecommendations(limit, callback)
        }
    }

    return {
        initial: initialMethod,
        tinderAuthentication: tinderAuthentication,
        authorize: authorize,
        sendMessage: sendMessage,
        getMessages: getMessages,
        getMatches: getMatches,
        getUserId: getUserId,
        setLocation: setLocation,
        like: like,
        unlike: unlike,
        getRecommendations: getRecommendations,
        isAuthorized: isAuthorized
    }
})()
